@@ -1,19 +1,17 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os
-from calibre.utils.run_tests import find_tests_in_dir, run_tests
+from calibre.utils.run_tests import find_tests_in_package, run_tests
 
 
 def find_tests():
-    base = os.path.dirname(os.path.abspath(__file__))
-    return find_tests_in_dir(base)
+    return find_tests_in_package('calibre.db.tests')
+
 
 if __name__ == '__main__':
     try:
