@@ -307,8 +307,8 @@ class TitleSortEdit(TitleEdit, ToMetadataMixin):
         self.setWhatsThis(tt)
 
     def auto_generate(self, *args):
-        self.current_val = title_sort(self.title_edit.current_val,
-                lang=self.book_lang)
+        self.set_value(title_sort(self.title_edit.current_val,
+                lang=self.book_lang))
 
     def break_cycles(self):
         try:
@@ -361,7 +361,7 @@ class AuthorsEdit(EditWithComplete, ToMetadataMixin):
         from calibre.gui2.dialogs.authors_edit import AuthorsEdit
         d = AuthorsEdit(all_authors, current_authors, self)
         if d.exec_() == d.Accepted:
-            self.current_val = d.authors
+            self.set_value(d.authors)
 
     def manage_authors(self):
         if self.original_val != self.current_val:
@@ -529,24 +529,24 @@ class AuthorSortEdit(EnLineEdit, ToMetadataMixin):
                     if meth in ('invert', 'nocomma', 'comma'):
                         one = rest.strip() + ' ' + ln.strip()
                 ans.append(one)
-            self.authors_edit.current_val = ans
+            self.authors_edit.set_value(ans)
 
     def auto_generate(self, *args):
         au = unicode_type(self.authors_edit.text())
         au = re.sub(r'\s+et al\.$', '', au).strip()
         authors = string_to_authors(au)
-        self.current_val = self.author_sort_from_authors(authors)
+        self.set_value(self.author_sort_from_authors(authors))
 
     def author_to_sort(self, *args):
         au = unicode_type(self.authors_edit.text())
         au = re.sub(r'\s+et al\.$', '', au).strip()
         if au:
-            self.current_val = au
+            self.set_value(au)
 
     def sort_to_author(self, *args):
         aus = self.current_val
         if aus:
-            self.authors_edit.current_val = [aus]
+            self.authors_edit.set_value([aus])
 
     def initialize(self, db, id_):
         self.current_val = db.author_sort(id_, index_is_id=True)
@@ -1094,7 +1094,7 @@ class Cover(ImageView):  # {{{
         self.dialog = parent
         self._cdata = None
         self.draw_border = False
-        self.cdata_before_trim = None
+        self.cdata_before_trim = self.cdata_before_generate = None
         self.cover_changed.connect(self.set_pixmap_from_data)
 
         class CB(RightClickButton):
@@ -1128,6 +1128,7 @@ class Cover(ImageView):  # {{{
         b.m = m = QMenu(b)
         b.setMenu(m)
         m.addAction(QIcon(I('config.png')), _('Customize the styles and colors of the generated cover'), self.custom_cover)
+        m.addAction(QIcon(I('edit-undo.png')), _('Undo last Generate cover'), self.undo_generate)
         b.setPopupMode(b.DelayedPopup)
         self.buttons = [self.select_cover_button, self.remove_cover_button,
                 self.trim_cover_button, self.download_cover_button,
@@ -1142,6 +1143,11 @@ class Cover(ImageView):  # {{{
             self.current_val = self.cdata_before_trim
             self.cdata_before_trim = None
 
+    def undo_generate(self):
+        if self.cdata_before_generate:
+            self.current_val = self.cdata_before_generate
+            self.cdata_before_generate = None
+
     def frame_resized(self, ev):
         sz = ev.size()
         self.frame_size = (sz.width()//3, sz.height())
@@ -1208,6 +1214,7 @@ class Cover(ImageView):  # {{{
     def generate_cover(self, *args):
         from calibre.ebooks.covers import generate_cover
         mi = self.dialog.to_book_metadata()
+        self.cdata_before_generate = self.current_val
         self.current_val = generate_cover(mi)
 
     def custom_cover(self):
