@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -13,6 +13,7 @@ from calibre.gui2.preferences.save_template_ui import Ui_Form
 from calibre.library.save_to_disk import FORMAT_ARG_DESCS, preprocess_template
 from calibre.utils.formatter import validation_formatter
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
+from polyglot.builtins import unicode_type
 
 
 class SaveTemplate(QWidget, Ui_Form):
@@ -33,12 +34,12 @@ class SaveTemplate(QWidget, Ui_Form):
                 ' by clicking the device icon and choosing "Configure this device".'))
         rows = []
         for var in variables:
-            rows.append(u'<tr><td>%s</td><td>&nbsp;</td><td>%s</td></tr>'%
+            rows.append('<tr><td>%s</td><td>&nbsp;</td><td>%s</td></tr>'%
                     (var, FORMAT_ARG_DESCS[var]))
-        rows.append(u'<tr><td>%s&nbsp;</td><td>&nbsp;</td><td>%s</td></tr>'%(
+        rows.append('<tr><td>%s&nbsp;</td><td>&nbsp;</td><td>%s</td></tr>'%(
             _('Any custom field'),
             _('The lookup name of any custom field (these names begin with "#").')))
-        table = u'<table>%s</table>'%(u'\n'.join(rows))
+        table = '<table>%s</table>'%('\n'.join(rows))
         self.template_variables.setText(table)
 
         self.field_metadata = field_metadata
@@ -74,7 +75,7 @@ class SaveTemplate(QWidget, Ui_Form):
         except Exception as err:
             error_dialog(self, _('Invalid template'),
                     '<p>'+_('The template %s is invalid:')%tmpl +
-                    '<br>'+str(err), show=True)
+                    '<br>'+unicode_type(err), show=True)
             return False
         return True
 
@@ -82,6 +83,6 @@ class SaveTemplate(QWidget, Ui_Form):
         self.opt_template.set_value(val)
 
     def save_settings(self, config, name):
-        val = unicode(self.opt_template.text())
+        val = unicode_type(self.opt_template.text())
         config.set(name, val)
         self.opt_template.save_history(self.option_name+'_template_history')
